@@ -26,7 +26,7 @@
 #![doc(test(attr(warn(rust_2018_idioms))))]
 // Not needed for 2018 edition and conflicts with `rust_2018_idioms`
 #![doc(test(no_crate_inject))]
-#![doc(html_root_url = "https://docs.rs/serde_with_macros/1.4.2")]
+#![doc(html_root_url = "https://docs.rs/serde_with_macros/1.5.0")]
 // Necessary to silence the warning about clippy::unknown_clippy_lints on nightly
 #![allow(renamed_and_removed_lints)]
 // Necessary for nightly clippy lints
@@ -45,15 +45,15 @@ extern crate proc_macro;
 mod utils;
 
 use crate::utils::{DeriveOptions, IteratorExt as _};
+use darling::util::Override;
 use darling::{Error as DarlingError, FromField, FromMeta};
 use proc_macro::TokenStream;
 use proc_macro2::{Span, TokenStream as TokenStream2};
 use quote::quote;
-use syn::parse::Parser;
 use syn::punctuated::Pair;
 use syn::spanned::Spanned;
 use syn::{
-    parse_macro_input, Attribute, AttributeArgs, DeriveInput, Error, Field, Fields,
+    parse_macro_input, parse_quote, AttributeArgs, DeriveInput, Error, Field, Fields,
     GenericArgument, ItemEnum, ItemStruct, Meta, NestedMeta, Path, PathArguments, ReturnType, Type,
 };
 
@@ -163,16 +163,15 @@ where
     }
 
     // Add a dummy derive macro which consumes (makes inert) all field attributes
-    let attr_tokens = quote!(
+    let consume_serde_as_attribute = parse_quote!(
         #[derive(#serde_with_crate_path::__private_consume_serde_as_attributes)]
     );
-    let consume_serde_as_attribute = Attribute::parse_outer.parse2(attr_tokens)?;
 
     // For each field in the struct given by `input`, add the `skip_serializing_if` attribute,
     // if and only if, it is of type `Option`
     if let Ok(mut input) = syn::parse::<ItemStruct>(input.clone()) {
         apply_on_fields(&mut input.fields, function)?;
-        input.attrs.extend(consume_serde_as_attribute);
+        input.attrs.push(consume_serde_as_attribute);
         Ok(quote!(#input))
     } else if let Ok(mut input) = syn::parse::<ItemEnum>(input) {
         let errors: Vec<DarlingError> = input
@@ -186,7 +185,7 @@ where
             })
             .collect();
         if errors.is_empty() {
-            input.attrs.extend(consume_serde_as_attribute);
+            input.attrs.push(consume_serde_as_attribute);
             Ok(quote!(#input))
         } else {
             Err(DarlingError::multiple(errors))
@@ -228,7 +227,7 @@ where
 /// ```
 ///
 /// The `skip_serializing_if` annotation is repetitive and harms readability.
-/// Instead the same struct can be written as:
+/// Instead, the same struct can be written as:
 ///
 /// ```rust
 /// # use serde::Serialize;
@@ -329,13 +328,10 @@ fn skip_serializing_none_add_attr_to_field(field: &mut Field) -> Result<(), Stri
             }
 
             // Add the `skip_serializing_if` attribute
-            let attr_tokens = quote!(
+            let attr = parse_quote!(
                 #[serde(skip_serializing_if = "Option::is_none")]
             );
-            let attrs = Attribute::parse_outer
-                .parse2(attr_tokens)
-                .expect("Static attr tokens should not panic");
-            field.attrs.extend(attrs);
+            field.attrs.push(attr);
         } else {
             // Warn on use of `serialize_always` on non-Option fields
             let has_attr = field
@@ -398,7 +394,7 @@ fn field_has_attribute(field: &Field, namespace: &str, name: &str) -> bool {
 
 /// Convenience macro to use the [`serde_as`] system.
 ///
-/// The [`serde_as`] system is designed as a more flexible alterative to serde's with-annotation.
+/// The [`serde_as`] system is designed as a more flexible alternative to serde's with-annotation.
 ///
 /// # Example
 ///
@@ -440,7 +436,7 @@ fn field_has_attribute(field: &Field, namespace: &str, name: &str) -> bool {
 /// # What this macro does
 ///
 /// The `serde_as` macro only serves a convenience function.
-/// All the steps it takes can easily applied manually, in case the cost of an attribute macro is deemed to high.
+/// All the steps it performs, can easily be done manually, in case the cost of an attribute macro is deemed to high.
 /// The functionality can best be described with an example.
 ///
 /// ```rust,ignore
@@ -454,10 +450,10 @@ fn field_has_attribute(field: &Field, namespace: &str, name: &str) -> bool {
 ///
 /// 1. All the placeholder type `_` will be replaced with `::serde_with::Same`.
 ///     The placeholder type `_` marks all the places where the types `Serialize` implementation should be used.
-///     In the example it means that the u32 values will be serialize by the `Serialize` implementation of u32.
+///     In the example, it means that the `u32` values will serialize with the `Serialize` implementation of `u32`.
 ///     The `Same` type implements `SerializeAs` whenever the underlying type implements `Serialize` and is used to make the two traits compatible.
 ///
-///     If you specify a custom path for `serde_with` via the `crate` attribute the path to the `Same` type will be altered accordingly.
+///     If you specify a custom path for `serde_with` via the `crate` attribute, the path to the `Same` type will be altered accordingly.
 /// 2. Wrap the type from the annotation inside a `::serde_with::As`.
 ///     In the above example we know have something like `::serde_with::As::<Vec<::serde_with::Same>>`.
 ///     The `As` type acts as the opposite of the `Same` type.
@@ -467,6 +463,9 @@ fn field_has_attribute(field: &Field, namespace: &str, name: &str) -> bool {
 ///     Similarly, `serialize_as` is translated to `serialize_with`.
 ///
 ///     The field attributes will be kept on the struct/enum such that other macros can use them too.
+/// 4. It searches `#[serde_as(as = ...)]` if there is a type named `BorrowCow` under any path.
+///     If `BorrowCow` is found, the attribute `#[serde(borrow)]` is added to the field.
+///     If `#[serde(borrow)]` or `#[serde(borrow = "...")]` is already present, this step will be skipped.
 ///
 /// After all these steps, the code snippet will have transformed into roughly this.
 ///
@@ -479,8 +478,8 @@ fn field_has_attribute(field: &Field, namespace: &str, name: &str) -> bool {
 /// }
 /// ```
 ///
-/// [`serde_as`]: https://docs.rs/serde_with/1.9.2/serde_with/guide/index.html
-/// [re-exporting `serde_as`]: https://docs.rs/serde_with/1.9.2/serde_with/guide/serde_as/index.html#re-exporting-serde_as
+/// [`serde_as`]: https://docs.rs/serde_with/1.10.0/serde_with/guide/index.html
+/// [re-exporting `serde_as`]: https://docs.rs/serde_with/1.10.0/serde_with/guide/serde_as/index.html#re-exporting-serde_as
 #[proc_macro_attribute]
 pub fn serde_as(args: TokenStream, input: TokenStream) -> TokenStream {
     #[derive(FromMeta, Debug)]
@@ -542,16 +541,19 @@ fn serde_as_add_attr_to_field(
 
     #[derive(FromField, Debug)]
     #[darling(attributes(serde), allow_unknown_fields)]
-    struct SerdeWithOptions {
+    struct SerdeOptions {
         #[darling(default)]
         with: Option<String>,
         #[darling(default)]
         deserialize_with: Option<String>,
         #[darling(default)]
         serialize_with: Option<String>,
+
+        #[darling(default)]
+        borrow: Option<Override<String>>,
     }
 
-    impl SerdeWithOptions {
+    impl SerdeOptions {
         fn has_any_set(&self) -> bool {
             self.with.is_some() || self.deserialize_with.is_some() || self.serialize_with.is_some()
         }
@@ -566,7 +568,7 @@ fn serde_as_add_attr_to_field(
         return Ok(());
     }
     let serde_as_options = SerdeAsOptions::from_field(field)?;
-    let serde_with_options = SerdeWithOptions::from_field(field)?;
+    let serde_options = SerdeOptions::from_field(field)?;
 
     let mut errors = Vec::new();
     if !serde_as_options.has_any_set() {
@@ -574,7 +576,7 @@ fn serde_as_add_attr_to_field(
     }
 
     // Check if there are any conflicting attributes
-    if serde_as_options.has_any_set() && serde_with_options.has_any_set() {
+    if serde_as_options.has_any_set() && serde_options.has_any_set() {
         errors.push(DarlingError::custom("Cannot combine `serde_as` with serde's `with`, `deserialize_with`, or `serialize_with`."));
     }
 
@@ -589,28 +591,38 @@ fn serde_as_add_attr_to_field(
     }
 
     let type_same = &syn::parse_quote!(#serde_with_crate_path::Same);
+    let type_borrowcow = &syn::parse_quote!(BorrowCow);
     if let Some(Ok(type_)) = serde_as_options.r#as {
+        // If the field is not borrowed yet, check if we need to borrow it.
+        if serde_options.borrow.is_none() && has_type_embedded(&type_, type_borrowcow) {
+            let attr_borrow = parse_quote!(#[serde(borrow)]);
+            field.attrs.push(attr_borrow);
+        }
+
         let replacement_type = replace_infer_type_with_type(type_, type_same);
         let attr_inner_tokens = quote!(#serde_with_crate_path::As::<#replacement_type>).to_string();
-        let attr_tokens = quote!(#[serde(with = #attr_inner_tokens)]);
-        let attrs = Attribute::parse_outer.parse2(attr_tokens)?;
-        field.attrs.extend(attrs);
+        let attr = parse_quote!(#[serde(with = #attr_inner_tokens)]);
+        field.attrs.push(attr);
     }
     if let Some(Ok(type_)) = serde_as_options.deserialize_as {
+        // If the field is not borrowed yet, check if we need to borrow it.
+        if serde_options.borrow.is_none() && has_type_embedded(&type_, type_borrowcow) {
+            let attr_borrow = parse_quote!(#[serde(borrow)]);
+            field.attrs.push(attr_borrow);
+        }
+
         let replacement_type = replace_infer_type_with_type(type_, type_same);
         let attr_inner_tokens =
             quote!(#serde_with_crate_path::As::<#replacement_type>::deserialize).to_string();
-        let attr_tokens = quote!(#[serde(deserialize_with = #attr_inner_tokens)]);
-        let attrs = Attribute::parse_outer.parse2(attr_tokens)?;
-        field.attrs.extend(attrs);
+        let attr = parse_quote!(#[serde(deserialize_with = #attr_inner_tokens)]);
+        field.attrs.push(attr);
     }
     if let Some(Ok(type_)) = serde_as_options.serialize_as {
         let replacement_type = replace_infer_type_with_type(type_, type_same);
         let attr_inner_tokens =
             quote!(#serde_with_crate_path::As::<#replacement_type>::serialize).to_string();
-        let attr_tokens = quote!(#[serde(serialize_with = #attr_inner_tokens)]);
-        let attrs = Attribute::parse_outer.parse2(attr_tokens)?;
-        field.attrs.extend(attrs);
+        let attr = parse_quote!(#[serde(serialize_with = #attr_inner_tokens)]);
+        field.attrs.push(attr);
     }
 
     Ok(())
@@ -723,11 +735,76 @@ fn replace_infer_type_with_type(to_replace: Type, replacement: &Type) -> Type {
     }
 }
 
-/// Deserialize value by using it's [`FromStr`] implementation
+/// Check if a type ending in the `syn::Ident` `embedded_type` is contained in `type_`.
+fn has_type_embedded(type_: &Type, embedded_type: &syn::Ident) -> bool {
+    match type_ {
+        // Base cases
+        Type::Infer(_) => false,
+        Type::Never(_inner) => false,
+
+        // Recursive cases
+        // Iterate through all positions where a type could occur and recursively call this function
+        Type::Array(inner) => has_type_embedded(&inner.elem, embedded_type),
+        Type::Group(inner) => has_type_embedded(&inner.elem, embedded_type),
+        Type::Paren(inner) => has_type_embedded(&inner.elem, embedded_type),
+        Type::Path(inner) => {
+            match inner.path.segments.last() {
+                Some(t) => {
+                    if t.ident == *embedded_type {
+                        return true;
+                    }
+
+                    match &t.arguments {
+                        PathArguments::None => false,
+                        PathArguments::AngleBracketed(inner) => {
+                            // Iterate over the args between the angle brackets
+                            inner
+                                .args
+                                .iter()
+                                .any(|generic_argument| match generic_argument {
+                                    // replace types within the generics list, but leave other stuff like lifetimes untouched
+                                    GenericArgument::Type(type_) => {
+                                        has_type_embedded(type_, embedded_type)
+                                    }
+                                    _ga => false,
+                                })
+                        }
+                        PathArguments::Parenthesized(inner) => {
+                            inner
+                                .inputs
+                                .iter()
+                                .any(|type_| has_type_embedded(type_, embedded_type))
+                                || match &inner.output {
+                                    ReturnType::Type(_arrow, type_) => {
+                                        has_type_embedded(type_, embedded_type)
+                                    }
+                                    _default => false,
+                                }
+                        }
+                    }
+                }
+                None => false,
+            }
+        }
+        Type::Ptr(inner) => has_type_embedded(&inner.elem, embedded_type),
+        Type::Reference(inner) => has_type_embedded(&inner.elem, embedded_type),
+        Type::Slice(inner) => has_type_embedded(&inner.elem, embedded_type),
+        Type::Tuple(inner) => inner.elems.pairs().any(|pair| match pair {
+            Pair::Punctuated(type_, _) | Pair::End(type_) => {
+                has_type_embedded(type_, embedded_type)
+            }
+        }),
+
+        // Pass unknown types or non-handleable types (e.g., bare Fn) without performing any replacements
+        _type_ => false,
+    }
+}
+
+/// Deserialize value by using its [`FromStr`] implementation
 ///
 /// This is an alternative way to implement `Deserialize` for types which also implement [`FromStr`] by deserializing the type from string.
 /// Ensure that the struct/enum also implements [`FromStr`].
-/// If the implementation is missing you will get a error message like
+/// If the implementation is missing, you will get an error message like
 /// ```text
 /// error[E0277]: the trait bound `Struct: std::str::FromStr` is not satisfied
 /// ```
@@ -783,7 +860,7 @@ fn replace_infer_type_with_type(to_replace: Type, replacement: &Type) -> Type {
 /// [`Display`]: std::fmt::Display
 /// [`FromStr`]: std::str::FromStr
 /// [cargo-toml-rename]: https://doc.rust-lang.org/cargo/reference/specifying-dependencies.html#renaming-dependencies-in-cargotoml
-/// [serde-as-crate]: https://docs.rs/serde_with/1.9.2/serde_with/guide/serde_as/index.html#re-exporting-serde_as
+/// [serde-as-crate]: https://docs.rs/serde_with/1.10.0/serde_with/guide/serde_as/index.html#re-exporting-serde_as
 /// [serde-crate]: https://serde.rs/container-attrs.html#crate
 #[proc_macro_derive(DeserializeFromStr, attributes(serde_with))]
 pub fn derive_deserialize_fromstr(item: TokenStream) -> TokenStream {
@@ -852,7 +929,7 @@ fn deserialize_fromstr(input: DeriveInput, serde_with_crate_path: Path) -> Token
 ///
 /// This is an alternative way to implement `Serialize` for types which also implement [`Display`] by serializing the type as string.
 /// Ensure that the struct/enum also implements [`Display`].
-/// If the implementation is missing you will get a error message like
+/// If the implementation is missing, you will get an error message like
 /// ```text
 /// error[E0277]: `Struct` doesn't implement `std::fmt::Display`
 /// ```
@@ -893,7 +970,7 @@ fn deserialize_fromstr(input: DeriveInput, serde_with_crate_path: Path) -> Token
 /// [`Display`]: std::fmt::Display
 /// [`FromStr`]: std::str::FromStr
 /// [cargo-toml-rename]: https://doc.rust-lang.org/cargo/reference/specifying-dependencies.html#renaming-dependencies-in-cargotoml
-/// [serde-as-crate]: https://docs.rs/serde_with/1.9.2/serde_with/guide/serde_as/index.html#re-exporting-serde_as
+/// [serde-as-crate]: https://docs.rs/serde_with/1.10.0/serde_with/guide/serde_as/index.html#re-exporting-serde_as
 /// [serde-crate]: https://serde.rs/container-attrs.html#crate
 #[proc_macro_derive(SerializeDisplay, attributes(serde_with))]
 pub fn derive_serialize_display(item: TokenStream) -> TokenStream {
@@ -922,7 +999,7 @@ fn serialize_display(input: DeriveInput, serde_with_crate_path: Path) -> TokenSt
             where
                 S: #serde_with_crate_path::serde::Serializer,
             {
-                serializer.serialize_str(&::std::string::ToString::to_string(self))
+                serializer.collect_str(&self)
             }
         }
     }
@@ -933,8 +1010,8 @@ fn serialize_display(input: DeriveInput, serde_with_crate_path: Path) -> TokenSt
 ///
 /// The only task of this derive macro is to consume any `serde_as` attributes and turn them into inert attributes.
 /// This allows the serde_as macro to keep the field attributes without causing compiler errors.
-/// The intend is that keeping the field attributes allows downstream crates to consume and akt on them without causing an ordering dependency to the serde_as macro.
-/// Otherwise, downstream proc-macros would need to be places *in front of* the main `#[serde_as]` attribute, since otherwise the field attributes would already be stripped off.
+/// The intend is that keeping the field attributes allows downstream crates to consume and act on them without causing an ordering dependency to the serde_as macro.
+/// Otherwise, downstream proc-macros would need to be placed *in front of* the main `#[serde_as]` attribute, since otherwise the field attributes would already be stripped off.
 ///
 /// More details about the use-cases in the Github discussion: <https://github.com/jonasbb/serde_with/discussions/260>.
 #[proc_macro_derive(__private_consume_serde_as_attributes, attributes(serde_as))]
